/* $Workfile: Dsp55Ioctl.h $   $Revision: 8 $    $Modtime: 3/20/03 1:46p $  */
#ifndef DSP55_IOCTL_H
#define DSP55_IOCTL_H
/*------------------------------------------------------- File Description -*\

  @doc PublicDsp55SdkApi

  Odin TeleSystems Inc.

  SAA-1012-1 OTX DSP C55x Software Development Kit (SDK)

  @module

    Dsp55Ioctl.h | Macros and structs for sharing data between a C55x DSP and the host.

  @contents2
  Dsp55Ioctl.h |

  @index enum struct type | Dsp55Ioctl

  @contents2
  Host-Dsp Communication addresses (Dsp55Ioctl.h) |

  Overriding the some of the macros defined in Dsp55Ioctl.h
  
  We keep the host/dsp communication buffers below address 0x10000 so that
  we can use unsigned short as the data type for the address pointers.

\*--------------------------------------------------------------------------*/


#ifdef HDR_DBG // Define to trace include order of headers
#pragma message("\n" __FILE__ "(1): Including\n")
#endif


#include "DspIoctl.h"

#if defined(__linux__) 
#pragma pack(2)
#elif defined(_TMS320C5XX)
// No pack pragma needed for the DSP compiler
#elif defined(__TMS320C55X__)
// No pack pragma needed for the DSP compiler
#else
#pragma pack(push,2)
#endif

// Note: the addresses in this file are byte addresses
// Note: the lengths in this file are specified in number of words (not bytes)


//#define OTXDSP55_SDK_INTERRUPT_CONFIRM_ADDR 0x0000C0   // The host writes a non-zero value to this address when he confirms the interrupt that was sent to him
 
// Asynchronous Notification Events from Dsp To Host 
// @type OTXDSP_SDK_EVENT_BUF_PROCESSING_ADDR |
#define OTXDSP_SDK_EVENT_BUF_PROCESSING_ADDR 0x0000C2   // The host writes the current processing address to this location

// @type OTXDSP_SDK_EVENT_BUF_ADDR |
#define OTXDSP_SDK_EVENT_BUF_ADDR            0x0000C4   // Start address for Circular buffer of event from the DSP to the host

// @type OTXDSP_SDK_EVENT_BUF_LEN |
#define OTXDSP_SDK_EVENT_BUF_LEN             (0x003E00/2)   // Length of the Circular buffer (in words)


// IO CONTROL from Host to Dsp
// @type OTXDSP_SDK_IOCTL_CODE_ADDR |
#define OTXDSP_SDK_IOCTL_CODE_ADDR         0x004010   // Start of the DSP IO Control packet. The host will always wait for the DSP to acknowledge the command (by writing <t OTXDSP_SDK_IO_NO_COMMAND> to this address).

// @type OTXDSP_SDK_IOCTL_HANDLE_ADDR |
#define OTXDSP_SDK_IOCTL_HANDLE_ADDR       (OTXDSP_SDK_IOCTL_CODE_ADDR + 2)   // Handle of the Logical device (nEventSourceId parameter of <f OtxDrvCreateLogicalDevice>)

// @type OTXDSP_SDK_IOCTL_TASKREF_ADDR |
#define OTXDSP_SDK_IOCTL_TASKREF_ADDR      (OTXDSP_SDK_IOCTL_CODE_ADDR + 4)  // Task Reference

// @type OTXDSP_SDK_IOCTL_DATA_LEN_ADDR |
#define OTXDSP_SDK_IOCTL_DATA_LEN_ADDR     (OTXDSP_SDK_IOCTL_CODE_ADDR + 6) // Data length for the command issued by the host

// @type OTXDSP_SDK_IOCTL_DATA_ADDR |
#define OTXDSP_SDK_IOCTL_DATA_ADDR         (OTXDSP_SDK_IOCTL_CODE_ADDR + 8)  // Start of data

#define OTXDSP_SDK_IOCTL_DATA_MAX_SIZE  ((0x008000 - OTXDSP_SDK_IOCTL_DATA_ADDR - 2)/2) // Max Data Length (in words)


#define OTXDSP_SDK_EVENT_BUF_INCLUDE_HANDLE 1
#define OTXDSP_SDK_EVENT_BUF_INCLUDE_TASKREF 1


#if defined(__linux__) 
#pragma pack(2)
#elif defined(_TMS320C5XX)
// No pack pragma needed for the DSP compiler
#elif defined(__TMS320C55X__)
// No pack pragma needed for the DSP compiler
#else
#pragma pack(pop)
#endif


/******************************************************************************

  Copyright (C) Odin TeleSystems Inc., 2002-2003


  All rights reserved. No part of this computer program may be reproduced,
  stored in any retrieval system, or transmitted, in any form or by any
  means, electronic, mechanical, photocopying, recording, or otherwise,
  without the prior written permission of Odin TeleSystems Inc., or without
  a permission stipulated in the agreement/contract under which this computer
  program was supplied.

******************************************************************************/
#endif /* DSP55_IOCTL_H */
